/* 
 *	Copyright (C) 2003-2004 Gabest
 *	http://www.gabest.org
 *
 *	(c) 2003-2004 by Peter Wimmer
 *  pwimmer@gmx.at
 *
 *  GplMpgDec.ax is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  GplMpgDec.ax is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *  http://www.gnu.org/copyleft/gpl.html
 *
 */

// Interface to configure GPL MPEG-1/2 Decoder

#ifdef __cplusplus
extern "C" {
#endif

// The GPL MPEG-1/2 Decoder exposes this interface. You can get it by
// using QueryInterface on the IBaseFilter interface you get when creating
// the filter.

// {B31FB86F-46C4-48b0-BB2B-6935CF404E35}
DEFINE_GUID(IID_IMpeg2DecFilter, 0xb31fb86f, 0x46c4, 0x48b0, 0xbb, 0x2b, 0x69, 0x35, 0xcf, 0x40, 0x4e, 0x35);

interface IMpeg2DecFilter:
  public IUnknown
{
  STDMETHOD(SetDeinterlaceMethod) (THIS_ int iMethod) PURE;
  STDMETHOD_(int, GetDeinterlaceMethod) (THIS) PURE;

  // Brightness: -255.0 to 255.0, default 0.0
  // Contrast: 0.0 to 10.0, default 1.0
  // Hue: -180.0 to +180.0, default 0.0
  // Saturation: 0.0 to 10.0, default 1.0
  STDMETHOD(SetBrightness) (THIS_ double bright) PURE;
  STDMETHOD(SetContrast) (THIS_ double cont) PURE;
  STDMETHOD(SetHue) (THIS_ double hue) PURE;
  STDMETHOD(SetSaturation) (THIS_ double sat) PURE;
  STDMETHOD_(double, GetBrightness) (THIS) PURE;
  STDMETHOD_(double, GetContrast) (THIS) PURE;
  STDMETHOD_(double, GetHue) (THIS) PURE;
  STDMETHOD_(double, GetSaturation) (THIS) PURE;

  STDMETHOD(EnableForcedSubtitles) (THIS_ bool fEnable) PURE;
  STDMETHOD_(bool, IsForcedSubtitlesEnabled) (THIS) PURE;

  STDMETHOD(EnablePlanarYUV) (THIS_ bool fEnable) PURE;
  STDMETHOD_(bool, IsPlanarYUVEnabled) (THIS) PURE;
};

#ifdef __cplusplus
}
#endif

#define DEINTERLACE_AUTO 0  // Will always be DIBlend for DVDs!
#define DEINTERLACE_WEAVE 1  // Use this for progressive sequences.
#define DEINTERLACE_BLEND 2  // Use this to deinterlace interlaced sequences using blend method.
#define DEINTERLACE_BOB 3  // Use this to deinterlace interlaced sequences using bob method.
#define DEINTERLACE_WEAVEEX 4  // Use this for interlaced sequences.

// Weave will interpolate color for each field separately. This is
// necessary for real interlaced content. Use WeaveEx if a progressive
// sequence was just encoded interlaced (e.g. DVDs).